@@ -33,6 +33,12 @@ Settings* Settings::instance()
     return instance_;
 }
 
+void Settings::close()
+{
+    delete instance_;
+    instance_ = 0;
+}
+
 bool Settings::set(QString const& name, QString const& value)
 {
     bool connected = DB::connected();
