@@ -23,6 +23,7 @@
 #include "calllistener.h"
 #include "settings.h"
 #include "cache.h"
+#include "contactmanager.h"
 
 namespace
 {
@@ -37,9 +38,9 @@ namespace
 
 QDBusConnection CallListener::systemBus_ = QDBusConnection::systemBus();
 
-CallListener::CallListener(): eniro_(0), contactManager_(0),
-connectionManager_(0), closeConnection_(false), box_(0), label_(0),
-retries_(-1)
+CallListener::CallListener(): eniro_(0), connectionManager_(0),
+closeConnection_(false), initialized_(false), box_(0), label_(0),
+retries_(-1), site_(Eniro::FI)
 {
 }
 
@@ -64,23 +65,8 @@ void CallListener::begin()
                        this,
                        SLOT(callTerminate()));
 
-    contactManager_ = new ContactManager;
-
-    eniro_ = new Eniro(Eniro::stringToSite(Settings::instance()->get("site")));
-
-    eniro_->setMaxResults(1);
-    eniro_->setFindNumber(false);
-    eniro_->setTimeout(REQUEST_TIMEOUT);
-
-    connect(eniro_, SIGNAL(requestFinished(QVector <Eniro::Result> const&,
-                                           Eniro::SearchDetails const&, bool)),
-                                           this, SLOT(requestFinished(QVector <Eniro::Result> const&,
-                                                                      Eniro::SearchDetails const&, bool)));
-
-    box_ = new InformationBox();
-    label_ = new QLabel("", box_);
-    label_->setMargin(8);
-    box_->setWidget(label_);
+    site_ = Eniro::stringToSite(Settings::instance()->get("site"));
+    Settings::close();
 
     qDebug() << "Starting...";
 
@@ -102,27 +88,23 @@ void CallListener::end()
                           this,
                           SLOT(callTerminate()));
 
-    delete contactManager_;
-    contactManager_ = 0;
-    delete connectionManager_;
-    connectionManager_ = 0;
-    delete eniro_;
-    eniro_ = 0;
-    delete box_;
-    box_ = 0;
-    delete label_;
-    label_ = 0;
+    searchClose();
+
 }
 
 void CallListener::search(Eniro::SearchDetails const& details)
 {
     qDebug() << "Search called";
 
+    searchInit();
+
     Eniro::Result result;
 
     if(Cache::instance().findItem(details.query, result))
     {
 
+        qDebug() << "Found from cache";
+
         showDelayedResult(createResult(result.name,
                                        result.street,
                                        result.city), BANNER_DELAY);
@@ -132,13 +114,7 @@ void CallListener::search(Eniro::SearchDetails const& details)
         retries_ = 0;
         currentSearch_ = details.query;
 
-        if(connectionManager_)
-        {
-            delete connectionManager_;
-            connectionManager_ = 0;
-        }
-
-        connectionManager_ = new ConnectionManager;
+        showDelayedResult(tr("Searching..."), BANNER_DELAY);
 
         if(!connectionManager_->isConnected())
         {
@@ -150,7 +126,8 @@ void CallListener::search(Eniro::SearchDetails const& details)
             closeConnection_ = false;
         }
 
-        showDelayedResult(tr("Searching..."), BANNER_DELAY);
+        qDebug() << "Starting to search...";
+
         eniro_->search(details);
     }
 
@@ -161,7 +138,7 @@ void CallListener::requestFinished(QVector <Eniro::Result> const& results,
                                    bool error)
 {
     // If box is not visible, the call must have been terminated already
-    if(!box_->isVisible())
+    if(!initialized_ || !box_->isVisible())
     {
         return;
     }
@@ -213,10 +190,6 @@ void CallListener::requestFinished(QVector <Eniro::Result> const& results,
         connectionManager_->disconnect(true);
         closeConnection_ = false;
     }
-
-    delete connectionManager_;
-    connectionManager_ = 0;
-
 }
 
 QString CallListener::createResult(QString const& name, QString const& street, QString const& city)
@@ -240,6 +213,11 @@ QString CallListener::createResult(QString const& name, QString const& street, Q
 
 void CallListener::showResult(QString const& text)
 {
+    if(!initialized_)
+    {
+        return;
+    }
+
     label_->setText("<font color='black'>" + text + "</font>");
 
     if(box_->isVisible())
@@ -254,7 +232,9 @@ void CallListener::incomingCall(QDBusObjectPath path, QString number)
 {
     qDebug() << "Incoming: " << number;
 
-    if(!contactManager_->numberExists(number))
+    ContactManager cm;
+
+    if(!cm.numberExists(number))
     {
         qDebug() << "Number doesn't exist";
 
@@ -265,8 +245,6 @@ void CallListener::incomingCall(QDBusObjectPath path, QString number)
                            this,
                            SLOT(callTerminate()));
 
-        qDebug() << "Going to search";
-
         search(Eniro::SearchDetails(number));
     }
     else
@@ -277,7 +255,7 @@ void CallListener::incomingCall(QDBusObjectPath path, QString number)
 
 void CallListener::callTerminate()
 {
-    if(box_->isVisible())
+    if(initialized_ && box_ && box_->isVisible())
     {
         box_->hide();
     }
@@ -288,8 +266,7 @@ void CallListener::callTerminate()
         closeConnection_ = false;
     }
 
-    delete connectionManager_;
-    connectionManager_ = 0;
+    searchClose();
 }
 
 void CallListener::showDelayedResult(QString const& text, int delay)
@@ -300,7 +277,7 @@ void CallListener::showDelayedResult(QString const& text, int delay)
 
 void CallListener::showTimedMessage()
 {
-    if(timedMessage_.size() == 0)
+    if(timedMessage_.size() == 0 || !initialized_)
     {
         return;
     }
@@ -309,3 +286,47 @@ void CallListener::showTimedMessage()
 
     timedMessage_ = "";
 }
+
+void CallListener::searchInit()
+{
+    qDebug() << "Initializing search...";
+
+    connectionManager_ = new ConnectionManager;
+
+    eniro_ = new Eniro(site_);
+    eniro_->setMaxResults(1);
+    eniro_->setFindNumber(false);
+    eniro_->setTimeout(REQUEST_TIMEOUT);
+
+    connect(eniro_, SIGNAL(requestFinished(QVector <Eniro::Result> const&,
+                                           Eniro::SearchDetails const&, bool)),
+                                           this, SLOT(requestFinished(QVector <Eniro::Result> const&,
+                                                                      Eniro::SearchDetails const&, bool)));
+    box_ = new InformationBox;
+    label_ = new QLabel("", box_);
+    label_->setMargin(8);
+    box_->setWidget(label_);
+    initialized_ = true;
+}
+
+void CallListener::searchClose()
+{
+    initialized_ = false;
+
+    qDebug() << "Closing search...";
+
+    if(eniro_)
+    {
+        disconnect(eniro_, SIGNAL(requestFinished(QVector <Eniro::Result> const&,
+                                                  Eniro::SearchDetails const&, bool)),
+                                                  this, SLOT(requestFinished(QVector <Eniro::Result> const&,
+                                                                             Eniro::SearchDetails const&, bool)));
+    }
+
+    delete connectionManager_;
+    connectionManager_ = 0;
+    delete eniro_;
+    eniro_ = 0;
+    delete box_;
+    box_ = 0;
+}
